@@ -4,7 +4,7 @@ mod pipeline_test;
 
 use crate::construction::heuristics::{ActivityContext, RouteContext, SolutionContext};
 use crate::models::common::Cost;
-use crate::models::problem::Job;
+use crate::models::problem::{Job, TargetConstraint};
 use hashbrown::HashSet;
 use std::slice::Iter;
 use std::sync::Arc;
@@ -83,9 +83,11 @@ pub trait ConstraintModule {
     /// Target route is defined by `route_index` which refers to `routes` collection in solution context.
     /// Inserted job is `job`.
     /// This method should call `accept_route_state` internally.
+    /// This method should NOT modify amount of job activities in the tour.
     fn accept_insertion(&self, solution_ctx: &mut SolutionContext, route_index: usize, job: &Job);
 
     /// Accept route and updates its state to allow more efficient constraint checks.
+    /// This method should NOT modify amount of job activities in the tour.
     fn accept_route_state(&self, ctx: &mut RouteContext);
 
     /// Accepts insertion solution context allowing to update job insertion data.
@@ -104,12 +106,18 @@ pub trait ConstraintModule {
 
 /// Provides the way to work with multiple constraints.
 pub struct ConstraintPipeline {
-    modules: Vec<Box<dyn ConstraintModule + Send + Sync>>,
-    state_keys: HashSet<i32>,
-    hard_route_constraints: Vec<Arc<dyn HardRouteConstraint + Send + Sync>>,
-    hard_activity_constraints: Vec<Arc<dyn HardActivityConstraint + Send + Sync>>,
-    soft_route_constraints: Vec<Arc<dyn SoftRouteConstraint + Send + Sync>>,
-    soft_activity_constraints: Vec<Arc<dyn SoftActivityConstraint + Send + Sync>>,
+    /// Pipeline modules.
+    pub modules: Vec<Arc<dyn ConstraintModule + Send + Sync>>,
+    /// Registered state keys.
+    pub state_keys: HashSet<i32>,
+    /// Hard route constraints.
+    pub hard_route_constraints: Vec<Arc<dyn HardRouteConstraint + Send + Sync>>,
+    /// Hard activity constraints.
+    pub hard_activity_constraints: Vec<Arc<dyn HardActivityConstraint + Send + Sync>>,
+    /// Soft route constraints.
+    pub soft_route_constraints: Vec<Arc<dyn SoftRouteConstraint + Send + Sync>>,
+    /// Soft activity constraints.
+    pub soft_activity_constraints: Vec<Arc<dyn SoftActivityConstraint + Send + Sync>>,
 }
 
 impl Default for ConstraintPipeline {
@@ -128,21 +136,53 @@ impl Default for ConstraintPipeline {
 impl ConstraintPipeline {
     /// Accepts job insertion.
     pub fn accept_insertion(&self, solution_ctx: &mut SolutionContext, route_index: usize, job: &Job) {
+        let activities = solution_ctx.routes.get_mut(route_index).unwrap().route.tour.activity_count();
         self.modules.iter().for_each(|c| c.accept_insertion(solution_ctx, route_index, job));
+        assert_eq!(activities, solution_ctx.routes.get_mut(route_index).unwrap().route.tour.activity_count());
+
         solution_ctx.routes.get_mut(route_index).unwrap().mark_stale(false)
     }
 
     /// Accepts route state.
     pub fn accept_route_state(&self, route_ctx: &mut RouteContext) {
         if route_ctx.is_stale() {
+            let activities = route_ctx.route.tour.activity_count();
             self.modules.iter().for_each(|c| c.accept_route_state(route_ctx));
+            assert_eq!(activities, route_ctx.route.tour.activity_count());
+
             route_ctx.mark_stale(false);
         }
     }
 
     /// Accepts solution state.
     pub fn accept_solution_state(&self, solution_ctx: &mut SolutionContext) {
-        self.modules.iter().for_each(|c| c.accept_solution_state(solution_ctx));
+        let _ = (0..).try_fold((usize::MAX, usize::MAX), |(required, ignored), counter| {
+            // NOTE if any job promotion occurs, then we might need to recalculate states.
+            // As it is hard to maintain dependencies between different modules, we reset process to
+            // beginning. However we do not expect recalculation to happen often, so this condition
+            // here is to prevent infinite loops and signalize about error in pipeline configuration
+            assert_ne!(counter, 100);
+
+            if required != solution_ctx.required.len() || ignored != solution_ctx.ignored.len() {
+                let required = solution_ctx.required.len();
+                let ignored = solution_ctx.ignored.len();
+
+                self.modules
+                    .iter()
+                    .try_for_each(|c| {
+                        c.accept_solution_state(solution_ctx);
+                        if required != solution_ctx.required.len() || ignored != solution_ctx.ignored.len() {
+                            Err(())
+                        } else {
+                            Ok(())
+                        }
+                    })
+                    .map(|_| (required, ignored))
+                    .or(Ok((usize::MAX, usize::MAX)))
+            } else {
+                Err(())
+            }
+        });
 
         solution_ctx.routes.iter_mut().for_each(|route_ctx| {
             route_ctx.mark_stale(false);
@@ -150,7 +190,7 @@ impl ConstraintPipeline {
     }
 
     /// Adds constraint module.
-    pub fn add_module(&mut self, module: Box<dyn ConstraintModule + Send + Sync>) -> &mut Self {
+    pub fn add_module(&mut self, module: TargetConstraint) -> &mut Self {
         module.state_keys().for_each(|key| {
             if let Some(duplicate) = self.state_keys.get(key) {
                 panic!("Attempt to register constraint with key duplication: {}", duplicate)
@@ -158,18 +198,23 @@ impl ConstraintPipeline {
             self.state_keys.insert(*key);
         });
 
-        module.get_constraints().for_each(|c| match c {
-            ConstraintVariant::HardRoute(c) => self.hard_route_constraints.push(c.clone()),
-            ConstraintVariant::HardActivity(c) => self.hard_activity_constraints.push(c.clone()),
-            ConstraintVariant::SoftRoute(c) => self.soft_route_constraints.push(c.clone()),
-            ConstraintVariant::SoftActivity(c) => self.soft_activity_constraints.push(c.clone()),
-        });
+        module.get_constraints().for_each(|constraint| Self::add_constraint(self, constraint));
 
         self.modules.push(module);
 
         self
     }
 
+    /// Adds constraint into pipeline.
+    pub fn add_constraint(&mut self, constraint: &ConstraintVariant) {
+        match constraint {
+            ConstraintVariant::HardRoute(c) => self.hard_route_constraints.push(c.clone()),
+            ConstraintVariant::HardActivity(c) => self.hard_activity_constraints.push(c.clone()),
+            ConstraintVariant::SoftRoute(c) => self.soft_route_constraints.push(c.clone()),
+            ConstraintVariant::SoftActivity(c) => self.soft_activity_constraints.push(c.clone()),
+        }
+    }
+
     /// Checks whether all hard route constraints are fulfilled.
     /// Returns result of first failed constraint or empty value.
     pub fn evaluate_hard_route(
@@ -200,6 +245,16 @@ impl ConstraintPipeline {
     pub fn evaluate_soft_activity(&self, route_ctx: &RouteContext, activity_ctx: &ActivityContext) -> Cost {
         self.soft_activity_constraints.iter().map(|c| c.estimate_activity(route_ctx, activity_ctx)).sum()
     }
+
+    /// Gets all constraint variants as a single iterator.
+    pub fn get_constraints(&self) -> impl Iterator<Item = ConstraintVariant> + '_ {
+        self.hard_route_constraints
+            .iter()
+            .map(|c| ConstraintVariant::HardRoute(c.clone()))
+            .chain(self.hard_activity_constraints.iter().map(|c| ConstraintVariant::HardActivity(c.clone())))
+            .chain(self.soft_route_constraints.iter().map(|c| ConstraintVariant::SoftRoute(c.clone())))
+            .chain(self.soft_activity_constraints.iter().map(|c| ConstraintVariant::SoftActivity(c.clone())))
+    }
 }
 
 impl PartialEq<RouteConstraintViolation> for RouteConstraintViolation {
