@@ -67,7 +67,7 @@ impl DynamicSelective {
     /// Creates a new instance of `DynamicSelective`.
     pub fn new_with_defaults(problem: Arc<Problem>, environment: Arc<Environment>) -> Self {
         let mutations = Self::get_mutations(problem, environment.clone());
-        let mutation_estimates = Self::get_estimates(mutations.clone());
+        let mutation_estimates = Self::get_estimates(mutations.as_slice());
 
         Self {
             heuristic_simulator: Simulator::new(
@@ -88,69 +88,109 @@ impl DynamicSelective {
         }
     }
 
-    fn get_mutations(problem: Arc<Problem>, environment: Arc<Environment>) -> Vec<Arc<dyn Mutation + Send + Sync>> {
-        let recreates: Vec<Arc<dyn Recreate + Send + Sync>> = vec![
-            Arc::new(RecreateWithSkipBest::new(1, 2)),
-            Arc::new(RecreateWithSkipBest::new(1, 4)),
-            Arc::new(RecreateWithRegret::new(1, 3)),
-            Arc::new(RecreateWithCheapest::default()),
-            Arc::new(RecreateWithPerturbation::new_with_defaults(environment.random.clone())),
-            Arc::new(RecreateWithGaps::default()),
-            Arc::new(RecreateWithBlinks::<SingleDimLoad>::new_with_defaults(environment.random.clone())),
-            Arc::new(RecreateWithBlinks::<MultiDimLoad>::new_with_defaults(environment.random.clone())),
-            Arc::new(RecreateWithFarthest::default()),
-            Arc::new(RecreateWithNearestNeighbor::default()),
-            Arc::new(RecreateWithSkipRandom::default_explorative_phased(Arc::new(RecreateWithCheapest::default()))),
+    fn get_mutations(
+        problem: Arc<Problem>,
+        environment: Arc<Environment>,
+    ) -> Vec<(Arc<dyn Mutation + Send + Sync>, String)> {
+        let recreates: Vec<(Arc<dyn Recreate + Send + Sync>, String)> = vec![
+            (Arc::new(RecreateWithSkipBest::new(1, 2)), "skip_best_1".to_string()),
+            (Arc::new(RecreateWithSkipBest::new(1, 4)), "skip_best_2".to_string()),
+            (Arc::new(RecreateWithRegret::new(1, 3)), "regret".to_string()),
+            (Arc::new(RecreateWithCheapest::default()), "cheapest".to_string()),
+            (
+                Arc::new(RecreateWithPerturbation::new_with_defaults(environment.random.clone())),
+                "perturbation".to_string(),
+            ),
+            (Arc::new(RecreateWithGaps::default()), "gaps".to_string()),
+            (
+                Arc::new(RecreateWithBlinks::<SingleDimLoad>::new_with_defaults(environment.random.clone())),
+                "blinks_single".to_string(),
+            ),
+            (
+                Arc::new(RecreateWithBlinks::<MultiDimLoad>::new_with_defaults(environment.random.clone())),
+                "blinks_multi".to_string(),
+            ),
+            (Arc::new(RecreateWithFarthest::default()), "farthest".to_string()),
+            (Arc::new(RecreateWithNearestNeighbor::default()), "nearest".to_string()),
+            (
+                Arc::new(RecreateWithSkipRandom::default_explorative_phased(Arc::new(RecreateWithCheapest::default()))),
+                "skip_random".to_string(),
+            ),
         ];
 
-        let primary_ruins: Vec<Arc<dyn Ruin + Send + Sync>> = vec![
-            Arc::new(AdjustedStringRemoval::default()),
-            Arc::new(NeighbourRemoval::default()),
-            Arc::new(WorstJobRemoval::default()),
-            Arc::new(ClusterRemoval::new_with_defaults(problem.clone(), environment.clone())),
-            Arc::new(RandomJobRemoval::new(RuinLimits::default())),
-            Arc::new(RandomRouteRemoval::default()),
+        let primary_ruins: Vec<(Arc<dyn Ruin + Send + Sync>, String)> = vec![
+            (Arc::new(AdjustedStringRemoval::default()), "asr".to_string()),
+            (Arc::new(NeighbourRemoval::default()), "neighbour_removal".to_string()),
+            (Arc::new(WorstJobRemoval::default()), "worst_job".to_string()),
+            (
+                Arc::new(ClusterRemoval::new_with_defaults(problem.clone(), environment.clone())),
+                "cluster_removal".to_string(),
+            ),
+            (Arc::new(RandomJobRemoval::new(RuinLimits::default())), "random_job_removal_1".to_string()),
+            (Arc::new(RandomRouteRemoval::default()), "random_route_removal".to_string()),
         ];
-        let secondary_ruins: Vec<Arc<dyn Ruin + Send + Sync>> = vec![
-            Arc::new(CloseRouteRemoval::default()),
-            Arc::new(RandomJobRemoval::new(RuinLimits::new(2, 8, 0.1, 2))),
+        let secondary_ruins: Vec<(Arc<dyn Ruin + Send + Sync>, String)> = vec![
+            (Arc::new(CloseRouteRemoval::default()), "close_route_removal".to_string()),
+            (Arc::new(RandomJobRemoval::new(RuinLimits::new(2, 8, 0.1, 2))), "random_job_removal_2".to_string()),
         ];
 
         // NOTE we need to wrap any of ruin methods in composite which calls restore context before recreate
-        let ruins =
-            primary_ruins
-                .iter()
-                .flat_map(|outer_ruin| {
-                    secondary_ruins.iter().map(move |inner_ruin| (outer_ruin.clone(), inner_ruin.clone()))
+        let ruins = primary_ruins
+            .iter()
+            .flat_map(|(outer_ruin, outer_name)| {
+                secondary_ruins.iter().map(move |(inner_ruin, inner_name)| {
+                    (outer_ruin.clone(), inner_ruin.clone(), format!("{}+{}", outer_name, inner_name))
                 })
-                .map::<Arc<dyn Ruin + Send + Sync>, _>(|(a, b)| Arc::new(CompositeRuin::new(vec![(a, 1.), (b, 1.)])))
-                .chain(primary_ruins.iter().chain(secondary_ruins.iter()).map::<Arc<dyn Ruin + Send + Sync>, _>(
-                    |ruin| Arc::new(CompositeRuin::new(vec![(ruin.clone(), 1.)])),
-                ))
-                .collect::<Vec<_>>();
+            })
+            .map::<(Arc<dyn Ruin + Send + Sync>, String), _>(|(a, b, name)| {
+                (Arc::new(CompositeRuin::new(vec![(a, 1.), (b, 1.)])), name)
+            })
+            .chain(primary_ruins.iter().chain(secondary_ruins.iter()).map::<(Arc<dyn Ruin + Send + Sync>, String), _>(
+                |(ruin, name)| (Arc::new(CompositeRuin::new(vec![(ruin.clone(), 1.)])), name.clone()),
+            ))
+            .collect::<Vec<_>>();
 
-        let mutations: Vec<Arc<dyn Mutation + Send + Sync>> = vec![
-            Arc::new(LocalSearch::new(Arc::new(ExchangeInterRouteBest::default()))),
-            Arc::new(LocalSearch::new(Arc::new(ExchangeInterRouteRandom::default()))),
-            Arc::new(LocalSearch::new(Arc::new(ExchangeIntraRouteRandom::default()))),
-            Arc::new(LocalSearch::new(Arc::new(RescheduleDeparture::default()))),
-            Arc::new(DecomposeSearch::new(StaticSelective::create_default_mutation(problem, environment), (2, 8), 4)),
+        let static_selective = StaticSelective::create_default_mutation(problem, environment);
+
+        let mutations: Vec<(Arc<dyn Mutation + Send + Sync>, String)> = vec![
+            (
+                Arc::new(LocalSearch::new(Arc::new(ExchangeInterRouteBest::default()))),
+                "local_exch_inter_route_best".to_string(),
+            ),
+            (
+                Arc::new(LocalSearch::new(Arc::new(ExchangeInterRouteRandom::default()))),
+                "local_exch_inter_route_random".to_string(),
+            ),
+            (
+                Arc::new(LocalSearch::new(Arc::new(ExchangeIntraRouteRandom::default()))),
+                "local_exch_intra_route_random".to_string(),
+            ),
+            (
+                Arc::new(LocalSearch::new(Arc::new(RescheduleDeparture::default()))),
+                "local_reschedule_departure".to_string(),
+            ),
+            (Arc::new(DecomposeSearch::new(static_selective.clone(), (2, 4), 4)), "decompose_search".to_string()),
+            (
+                Arc::new(InfeasibleSearch::new(static_selective, 4, (0.05, 0.2), (0.05, 0.33))),
+                "infeasible_search".to_string(),
+            ),
         ];
 
-        let mutations = recreates
+        recreates
             .iter()
-            .flat_map(|recreate| {
-                ruins.iter().map::<Arc<dyn Mutation + Send + Sync>, _>(move |ruin| {
-                    Arc::new(RuinAndRecreate::new(ruin.clone(), recreate.clone()))
+            .flat_map(|(recreate, recreate_name)| {
+                ruins.iter().map::<(Arc<dyn Mutation + Send + Sync>, String), _>(move |(ruin, ruin_name)| {
+                    (
+                        Arc::new(RuinAndRecreate::new(ruin.clone(), recreate.clone())),
+                        format!("{}+{}", ruin_name, recreate_name),
+                    )
                 })
             })
             .chain(mutations.into_iter())
-            .collect::<Vec<_>>();
-
-        mutations
+            .collect::<Vec<_>>()
     }
 
-    fn get_estimates(mutations: Vec<Arc<dyn Mutation + Send + Sync>>) -> ActionEstimates<SearchState> {
+    fn get_estimates(mutations: &[(Arc<dyn Mutation + Send + Sync>, String)]) -> ActionEstimates<SearchState> {
         let mutation_estimates = (0..mutations.len())
             .map(|idx| (SearchAction::Mutate { mutation_index: idx }, 0.))
             .collect::<HashMap<_, _>>();
@@ -197,7 +237,7 @@ enum SearchAction {
 }
 
 struct SearchActionRegistry {
-    pub mutations: Vec<Arc<dyn Mutation + Send + Sync>>,
+    pub mutations: Vec<(Arc<dyn Mutation + Send + Sync>, String)>,
 }
 
 struct SearchAgent<'a> {
@@ -222,7 +262,7 @@ impl<'a> Agent<SearchState> for SearchAgent<'a> {
         let new_individual = match action {
             SearchAction::Mutate { mutation_index } => {
                 let individual = self.individual.as_ref().unwrap();
-                let mutation = &self.registry.mutations[*mutation_index];
+                let (mutation, _) = &self.registry.mutations[*mutation_index];
 
                 mutation.mutate(self.refinement_ctx, individual)
             }
@@ -259,6 +299,8 @@ impl<'a> Agent<SearchState> for SearchAgent<'a> {
     }
 }
 
+impl<'a> SearchAgent<'a> {}
+
 fn try_exchange_estimates(heuristic_simulator: &mut Simulator<SearchState>) {
     let (best_known_max, diverse_state_max) = {
         let state_estimates = heuristic_simulator.get_state_estimates();
@@ -284,6 +326,6 @@ fn compare_to_best(refinement_ctx: &RefinementContext, individual: &Individual)
         .population
         .ranked()
         .next()
-        .map(|(best_known, _)| refinement_ctx.problem.objective.total_order(&individual, best_known))
+        .map(|(best_known, _)| refinement_ctx.problem.objective.total_order(individual, best_known))
         .unwrap_or(Ordering::Less)
 }
