@@ -54,7 +54,7 @@ impl StaticSelective {
 
         Self::new(vec![
             (
-                Arc::new(DecomposeSearch::new(default_mutation.clone(), (2, 8), 4)),
+                Arc::new(DecomposeSearch::new(default_mutation.clone(), (2, 4), 4)),
                 create_context_mutation_probability(
                     300,
                     10,
@@ -63,19 +63,26 @@ impl StaticSelective {
                 ),
             ),
             (local_search.clone(), create_scalar_mutation_probability(0.05, environment.random.clone())),
-            (default_mutation, create_scalar_mutation_probability(1., environment.random.clone())),
+            (default_mutation.clone(), create_scalar_mutation_probability(1., environment.random.clone())),
             (local_search, create_scalar_mutation_probability(0.05, environment.random.clone())),
+            (
+                Arc::new(InfeasibleSearch::new(default_mutation, 4, (0.05, 0.2), (0.05, 0.33))),
+                create_scalar_mutation_probability(0.01, environment.random.clone()),
+            ),
         ])
     }
 
     fn mutate(&self, refinement_ctx: &RefinementContext, insertion_ctx: &InsertionContext) -> InsertionContext {
         unwrap_from_result(
-            self.mutation_group.iter().filter(|(_, probability)| probability(refinement_ctx, insertion_ctx)).try_fold(
-                insertion_ctx.deep_copy(),
-                |ctx, (mutation, _)| {
+            self.mutation_group
+                .iter()
+                .filter(|(_, probability)| probability(refinement_ctx, insertion_ctx))
+                // NOTE not more than two mutations in a row
+                .take(2)
+                .try_fold(insertion_ctx.deep_copy(), |ctx, (mutation, _)| {
                     let new_insertion_ctx = mutation.mutate(refinement_ctx, &ctx);
 
-                    if refinement_ctx.problem.objective.total_order(&insertion_ctx, &new_insertion_ctx)
+                    if refinement_ctx.problem.objective.total_order(insertion_ctx, &new_insertion_ctx)
                         == Ordering::Greater
                     {
                         // NOTE exit immediately as we don't want to lose improvement from original individual
@@ -83,8 +90,7 @@ impl StaticSelective {
                     } else {
                         Ok(new_insertion_ctx)
                     }
-                },
-            ),
+                }),
         )
     }
 
