@@ -7,14 +7,10 @@ mod telemetry_test;
 use crate::algorithms::nsga2::Objective;
 use crate::construction::heuristics::InsertionContext;
 use crate::solver::population::SelectionPhase;
-use crate::solver::{RefinementContext, Statistics};
-use crate::utils::Timer;
+use crate::solver::{RefinementContext, RefinementSpeed, Statistics};
+use crate::utils::{InfoLogger, Timer};
 use std::fmt::Write;
 use std::ops::Deref;
-use std::sync::Arc;
-
-/// A logger type which is called with various information regarding the work done by the VRP solver.
-pub type InfoLogger = Arc<dyn Fn(&str)>;
 
 /// Encapsulates different measurements regarding algorithm evaluation.
 pub struct Metrics {
@@ -107,6 +103,7 @@ pub struct Telemetry {
     time: Timer,
     mode: TelemetryMode,
     improvement_tracker: ImprovementTracker,
+    speed_tracker: SpeedTracker,
     next_generation: Option<usize>,
 }
 
@@ -118,6 +115,7 @@ impl Telemetry {
             metrics: Metrics { duration: 0, generations: 0, speed: 0.0, evolution: vec![] },
             mode,
             improvement_tracker: ImprovementTracker::new(1000),
+            speed_tracker: SpeedTracker::default(),
             next_generation: None,
         }
     }
@@ -157,10 +155,12 @@ impl Telemetry {
 
         self.metrics.generations = generation;
         self.improvement_tracker.track(generation, is_improved);
+        self.speed_tracker.track(generation, termination_estimate);
 
         refinement_ctx.statistics = Statistics {
             generation,
             time: self.time.clone(),
+            speed: self.speed_tracker.get_current_speed(),
             improvement_all_ratio: self.improvement_tracker.i_all_ratio,
             improvement_1000_ratio: self.improvement_tracker.i_1000_ratio,
             termination_estimate,
@@ -186,12 +186,12 @@ impl Telemetry {
 
             if should_log_best {
                 self.log_individual(
-                    &self.get_individual_metrics(refinement_ctx, &best_individual, rank),
+                    &self.get_individual_metrics(refinement_ctx, best_individual, rank),
                     Some((refinement_ctx.statistics.generation, generation_time)),
                 )
             }
 
-            self.on_population(&refinement_ctx, should_log_population, should_track_population, should_dump_population);
+            self.on_population(refinement_ctx, should_log_population, should_track_population, should_dump_population);
         } else {
             self.log("no progress yet");
         }
@@ -226,11 +226,11 @@ impl Telemetry {
         let individuals = refinement_ctx
             .population
             .ranked()
-            .map(|(insertion_ctx, rank)| self.get_individual_metrics(refinement_ctx, &insertion_ctx, rank))
+            .map(|(insertion_ctx, rank)| self.get_individual_metrics(refinement_ctx, insertion_ctx, rank))
             .collect::<Vec<_>>();
 
         if should_log_population {
-            individuals.iter().for_each(|metrics| self.log_individual(&metrics, None));
+            individuals.iter().for_each(|metrics| self.log_individual(metrics, None));
             if should_dump_population {
                 self.log(&format!("\t{}", Self::get_population_state(refinement_ctx)));
             }
@@ -394,3 +394,32 @@ impl ImprovementTracker {
         self.i_1000_ratio = (improvements as f64) / ((generation + 1).min(self.buffer.len()) as f64);
     }
 }
+
+struct SpeedTracker {
+    initial: f64,
+    speed: RefinementSpeed,
+}
+
+impl Default for SpeedTracker {
+    fn default() -> Self {
+        Self { initial: 0., speed: RefinementSpeed::Moderate }
+    }
+}
+
+impl SpeedTracker {
+    pub fn track(&mut self, generation: usize, termination_estimate: f64) {
+        if generation == 0 {
+            self.initial = termination_estimate;
+        } else {
+            let delta = (termination_estimate - self.initial).max(0.);
+
+            if generation < 200 && delta > 0.1 {
+                self.speed = RefinementSpeed::Slow;
+            }
+        }
+    }
+
+    pub fn get_current_speed(&self) -> RefinementSpeed {
+        self.speed.clone()
+    }
+}
